function [ out ] = set_based_guidance_controller(nu, eta, u_d, psi_d, t  )

global prev_mode wp_oa num_obs 
global psi_B u_B x0_B y0_B


R_s = 70; %safety radius 
R_ss = 20;

mode = prev_mode;

p_ASV = [eta(1); eta(2)];
v_ASV = [nu(1); nu(2)];

x = eta(1); y = eta(2);


if num_obs > 0
    
    p_obs_mat = zeros(2,num_obs);
    x_obs_mat = zeros(1,num_obs);
    y_obs_mat = zeros(1,num_obs);
    u_obs_mat = zeros(1,num_obs);
    v_obs_mat = zeros(2,num_obs);
    psi_obs_mat = zeros(1,num_obs);
    sigma_mat = zeros(1,num_obs);
    
    p_obs_mat(:,1) = [x0_B + t*u_B*cos(psi_B); y0_B + t*u_B*sin(psi_B)];
    x_obs_mat(1) = p_obs_mat(1,1); 
    y_obs_mat(1) = p_obs_mat(2,1);
    u_obs_mat(1) = u_B;
    v_obs_mat(:,1) = [u_B*cos(psi_B); u_B*sin(psi_B)];
    psi_obs_mat(1) = psi_B;
    sigma_mat(1) = sqrt((x-x_obs_mat(1))^2+(y-y_obs_mat(1))^2);

    if num_obs > 1
        p_obs_mat(:,2) = [x0_C + t*u_C*cos(psi_C); y0_C + t*u_C*sin(psi_C)];
        x_obs_mat(2) = p_obs_mat(1,2); 
        y_obs_mat(2)= p_obs_mat(2,2);
        u_obs_mat(2) = u_C;
        v_obs_mat(:,2) = [u_C*cos(psi_C); u_C*sin(psi_C)];
        psi_obs_mat(2) = psi_C;
        sigma_mat(2) = sqrt((x-x_obs_mat(2))^2+(y-y_obs_mat(2))^2);
    end
    closest_obs = 1;
    for i=1:length(sigma_mat)
        if sigma_mat(i) < sigma_mat(closest_obs)
            closest_obs = i;
        end
    end
    p_obs = p_obs_mat(:,closest_obs);
    x_obs = x_obs_mat(closest_obs);
    y_obs = y_obs_mat(closest_obs);
    u_obs = u_obs_mat(closest_obs);
    v_obs = v_obs_mat(:,closest_obs);
    psi_obs = psi_obs_mat(closest_obs);
    sigma = sigma_mat(closest_obs);
    if sigma <= R_s
        clear_path = find_clear_path(p_ASV,p_obs,u_obs,psi_obs,R_ss);
        if clear_path == 1 && sigma > R_ss
                mode = 1; % path_following
                wp_oa = [];
                %disp('clear path')
            
        elseif prev_mode == 1 % path_following
            if sigma < R_s-1
                mode = 2; % obstacle avoidance
                %disp('switch to oa');
                new_wp(p_ASV,v_ASV,p_obs,v_obs,psi_obs,R_s,R_ss);
            else
                mode = 1;
            end
        end
    elseif sigma > R_s
        if prev_mode == 1 % path_following
            mode = 1; % path_following
        elseif prev_mode == 2 % obstacle avoidance
            clear_path = find_clear_path(p_ASV,p_obs,u_obs,psi_obs,R_s);
            sigma_dot = find_sigma_dot(x,y,nu,x_obs,y_obs,v_obs,psi_obs,sigma);
            if clear_path == 1 && prev_mode == 2
                mode = 1; % path_following
                wp_oa = [];
                %disp('clear path')
                %disp('switch to pf');

            elseif sigma_dot >= 0
                mode = 1; % path_following
                wp_oa = [];
                %disp('switch to pf');
            elseif sigma_dot < 0
                mode = 2; % obstacle avoidance
            end
        end
    end   
end
prev_mode = mode;
out = [u_d, psi_d];

end

